#include <JTC/JTC.h>
#include <iostream>

using namespace std;

enum SPORTELLO {A = 0, B = 1};

class Segreteria : public JTCMonitor
{
	bool occupato[2];		//A = 0, B = 1
	int disabiliInCoda;	    //numero di disabili in attesa
	SPORTELLO sportelloLibero()
	{
		if ( occupato[B] ) // cerca di occupare preferenzialmente B
			return A;	
		else return B;
	}

public:
    Segreteria()
    {
        occupato[A] = false;
        occupato[B] = false;
        disabiliInCoda = 0;
    }
	SPORTELLO vaiAlloSportello(bool disabile)
	{
		JTCSynchronized sync(*this);
        if ( disabile )
            disabiliInCoda++;
		while ( !disabile && ((occupato[A] && occupato[B]) || 
                (!occupato[A] && occupato[B] && disabiliInCoda > 0)) || 
			    (disabile && (occupato[0] && occupato[1])) )
			wait();
		if ( disabile )
			disabiliInCoda--;
		SPORTELLO sportello = sportelloLibero();
		occupato[sportello] = true;
		return sportello;
	}
	void lasciaLoSportello(SPORTELLO sportello)
	{
		JTCSynchronized sync(*this);
		occupato[sportello] = false;
		notifyAll();
	}
};	

Segreteria segreteria;

class Studente : public JTCThread
{
	int id;
	bool disabile;

public:
	Studente(int i, bool d)
	{
		id = i;
		disabile = d;
	}
	virtual void run()
	{
		while(true) 
        {
			cout << "Lo Studente " << id
                 << (disabile ? " + " : " - ") 
                 << "e' in fila." << endl;
			SPORTELLO sportello = segreteria.vaiAlloSportello(disabile);
			
            cout << "Lo Studente " << id 
                 << (disabile ? " + " : " - ") 
                 << "viene servito dallo Sportello " 
                 << (sportello == A ? "A." : "B.") << endl;
			sleep(rand() % 1000);
			
            cout << "Lo Studente " << id 
                 << (disabile ? " + " : " - ") 
                 << "lascia lo Sportello " 
                 << (sportello == A ? "A." : "B.") << endl;
			segreteria.lasciaLoSportello(sportello);
			
            sleep(rand() % 1000);
		}
	}
};

int main(int argc, char* argv[])
{
	JTCInitialize initialize;
	Studente* s[10];
    for (int i = 0; i < 6; i++) 
    {
		s[i] = new Studente(i, false);
		s[i]->start();
	}
    for (int i = 6; i < 10; i++) 
    {
		s[i] = new Studente(i, true);
		s[i]->start();
	}
	return 0;	
}
